@@ -1,17 +1,20 @@
 from argparse import ArgumentParser
 from textwrap import dedent
 from .arguments import arg
+import os
 
 import logging, sys, argparse
 
 logger = logging.getLogger('argdeco.command_decorator')
-logger.setLevel(logging.ERROR)
 
 try:
     basestring
 except NameError:
     basestring = str
 
+class Undefined:
+    pass
+
 class CommandDecorator:
     """
     Create a decorator to decorate functions with their arguments.
@@ -88,6 +91,8 @@ class CommandDecorator:
 
         if 'description' not in kwargs and 'epilog' not in kwargs:
             kwargs[self.doc] = sys._getframe().f_back.f_globals.get('__doc__')
+            if kwargs[self.doc] is not None:
+                kwargs[self.doc] = dedent(kwargs[self.doc])
 
         if 'formatter_class' not in kwargs:
             kwargs['formatter_class'] = self.formatter_class
@@ -119,6 +124,8 @@ class CommandDecorator:
             if isinstance(a, arg):
                 a.apply(self.argparser, self, self.get_name())
 
+        self.children = []
+
 
     def has_action(self):
         if self.commands is not None:
@@ -136,7 +143,20 @@ class CommandDecorator:
         if self.commands is None:
             raise KeyError(name)
 
-        return self.commands._name_parser_map[name]
+        cmd = self
+        items = name.split('.')
+        path, name = items[:-1], items[-1]
+
+        for k in path:
+            _map = dict((c.name, c) for c in cmd.children)
+            if k not in _map:
+                raise KeyError(name)
+            cmd = _map[k]
+
+        return cmd.commands._name_parser_map[name]
+
+    def get_action(self, name):
+        return self[name].get_default('action')
 
     def add_subcommands(self, command, *args, **kwargs):
         """add subcommands.
@@ -162,18 +182,24 @@ class CommandDecorator:
         try:
             cmd = self[command]
         except KeyError:
+            if 'formatter_class' not in kwargs:
+                kwargs['formatter_class'] = self.formatter_class
+
             cmd = self.add_parser(command, *args, **kwargs)
             args, kwargs = tuple(), dict()
 
         if subcommands is not None:
             kwargs = subcommands
 
-        return CommandDecorator(
+        child = CommandDecorator(
             argparser = self.argparser,
             commands  = cmd.add_subparsers(*args, **kwargs),
             parent = self,
             name   = command,
             )
+        self.children.append(child)
+
+        return child
 
     def update(self, command=None, **kwargs):
         """update data, which is usually passed in ArgumentParser initialization
@@ -189,7 +215,7 @@ class CommandDecorator:
             setattr(argparser, k, v)
 
     def add_argument(self, *args, **kwargs):
-        logger.info("add_argument: %s, %s", args, kwargs)
+        logger.debug("add_argument: %s, %s", args, kwargs)
         if len(args) == 1 and isinstance(args[0], arg):
             self.add_arguments(*args)
         else:
@@ -197,10 +223,16 @@ class CommandDecorator:
 
     # def register_config_map(self, dest):
     def register_config_map(self, context, dest, config_name):
-        if context not in self.config_map:
-            self.config_map[context] = {}
+        logger.debug("register_config_map: context=%s, dest=%s, config_name=%s", context, dest, config_name)
 
-        self.config_map[context][dest] = config_name
+        _root = self
+        while _root.parent is not None:
+            _root = _root.parent
+
+        if context not in _root.config_map:
+            _root.config_map[context] = {}
+
+        _root.config_map[context][dest] = config_name
 
     #     map_name = self.get_name()
     #     logger.debug("map_name=%s", map_name)
@@ -213,12 +245,29 @@ class CommandDecorator:
     #         self.config_map[map_name] = {}
 
 
-    def add_arguments(self, *args):
+    def add_arguments(self, *args, **kwargs):
+        context = kwargs.get('context')
+        argparser = kwargs.get('argparser')
+        if context is None:
+            context = self.get_name()
+
+        if argparser is None:
+            argparser = self.argparser
+
         for a in args:
-            a.apply(self.argparser, self, self.get_name())
+            if isinstance(a, basestring):
+                a = arg(a)
+            elif isinstance(a, dict):
+                a = arg(**a)
+            elif isinstance(a, (tuple,list)):
+                a = arg(*a)
+            elif not isinstance(a, arg):
+                raise ValueError("cannot convert %s into arg type" % a)
+
+            a.apply(argparser, self, context=context)
 
 
-    def get_config_name(self, action, name):
+    def get_config_name(self, action, name=None):
         '''get the name for configuration
 
         This returns a name respecting commands and subcommands.  So if you
@@ -228,19 +277,46 @@ class CommandDecorator:
         "index.ls.all" as configuration name for this option.
         '''
 
-        #import rpdb2 ; rpdb2.start_embedded_debugger('foo')
+        _name = None
 
-        _name = action.argdeco_name
-        config_name = None
-        while _name:
-            if _name not in self.config_map: continue
-            if name in self.config_map[_name]:
-                config_name = self.config_map[_name][name]
+        if name is None:
+            if '.' in action:
+                action, name = action.rsplit('.', 1)
+            else:
+                _name = ''
+                name = action
+
+        if _name is None:
+            if isinstance(action, basestring):
+                action = self.get_action(action)
+            _name = action.argdeco_name
+
+        logger.debug("_name=%s", _name)
+
+        config_name = Undefined
+        while True:
+            logger.debug("check _name=%s, name=%s", repr(_name), repr(name))
+
+            if _name in self.config_map:
+                if name in self.config_map[_name]:
+                    config_name = self.config_map[_name][name]
+                    if config_name is not None:
+                        if config_name.startswith('.'):
+                            config_name = _name + config_name
+                    break
+
+            if _name == '':
                 break
+
+            if '.' not in _name:
+                _name = ''
+                continue
+
             _name = _name.rsplit('.', 1)[0]
 
-        assert config_name, "could not determine config name for %s" % k
-
+        assert config_name is not Undefined, "could not determine config name for %s" % name
+#        if config_name.startswith('.'):
+#            config_name = config_name[1:]
         return config_name
 
 
@@ -267,6 +343,7 @@ class CommandDecorator:
         return '.'.join(reversed(path))
 
     def __call__(self, *args, **opts):
+
         def factory(func):
             _args = args
             if func.__doc__ is not None:
@@ -303,37 +380,24 @@ class CommandDecorator:
                 name = None
                 command = self.argparser
 
-            func.argdeco_name = self.get_name(name)
-
+            func.argdeco_name = context = self.get_name(name)
             self.config_map[func.argdeco_name] = {}
 
-            for a in _args:
-                if isinstance(a, arg):
-                    a.apply(command, self, context=func.argdeco_name)
-
-                else:
-                    command.add_argument(a)
+            self.add_arguments(*_args, argparser=command, context=context)
 
             command.set_defaults(action=func)
-            #self.func_map[id(func)] = name
+
             return func
 
         return factory
 
-    #def process_args(self, ):
+    def compile_args(self, argv=None, compile=None, preprocessor=None,
+        compiler_factory=None):
 
+        logger.debug(
+            "argv=%s, compile=%s, preprocessor=%s, compiler_factory=%s",
+            argv, compile, preprocessor, compiler_factory)
 
-
-    def execute(self, argv=None, compile=None, preprocessor=None, compiler_factory=None):
-        """Parse arguments and execute decorated function
-
-        argv: list of arguments
-        compile:
-            - None, pass args as keyword args to function
-            - True, pass args as single dictionary
-            - function, get args from parse_args() and return a pair of
-              tuple and dict to be passed as args and kwargs to function
-        """
         if argv is None:
             argv = sys.argv[1:]
 
@@ -350,6 +414,14 @@ class CommandDecorator:
         assert not (compiler_factory and compile), \
             "you can either define a compiler factory or a compile function"
 
+#        if 'BASH' in os.environ:
+#            if '_python_argcomplete_run' not in os.environ:
+#                logger.warning("Autocomplete is not activated.  See https://github.com/kislyuk/argcomplete#activating-global-completion for activating")
+        if compiler_factory:
+            compile = compiler_factory(self)
+
+        import argcomplete
+        argcomplete.autocomplete(self.argparser)
         args = self.argparser.parse_args(argv)
 
         if preprocessor:
@@ -363,37 +435,12 @@ class CommandDecorator:
         opts = vars(args).copy()
         del opts['action']
 
-        if compiler_factory:
-            compile = compiler_factory(self)
-
-        # if compiler_factory:
-        #     def _compile(args, **opts):
-        #         cfg = self.compiler_factory(args)
-        #
-        #         for k,v in opts.items():
-        #             _name = args.action.argdeco_name
-        #             config_name = None
-        #             while _name:
-        #                 if _name not in self.config_map: continue
-        #                 if k in self.config_map[_name]:
-        #                     config_name = self.config_map[_name][k]
-        #                     break
-        #                 _name = _name.rsplit('.', 1)[0]
-        #
-        #             assert config_name, "could not determine config name for %s" % k
-        #
-        #             self.compiler_factory[config_name] = v
-        #
-        #         return cfg
-        #
-        #     compile = _compile
-
         if compile is None or compile == 'kwargs':
-            return args.action(**opts)
+            (_args, _kwargs) = tuple(), opts
         elif compile is True or compile == 'dict':
-            return args.action(opts)
+            (_args, _kwargs) = (opts,), dict()
         elif compile == 'args':
-            return args.action(args)
+            (_args, _kwargs) = (args,), dict()
         else:
             compiled = compile(args, **opts)
 
@@ -406,9 +453,24 @@ class CommandDecorator:
             else:
                 raise "Unkown compilation: %s" % compiled
 
-            logger.debug("compiled: %s", compiled)
+        #logger.debug("compiled: %s", (_args, _kwargs))
+        return (args.action, _args, _kwargs)
+
+
+    def execute(self, argv=None, compile=None, preprocessor=None, compiler_factory=None):
+        """Parse arguments and execute decorated function
+
+        argv: list of arguments
+        compile:
+            - None, pass args as keyword args to function
+            - True, pass args as single dictionary
+            - function, get args from parse_args() and return a pair of
+              tuple and dict to be passed as args and kwargs to function
+        """
+
+        action, args, kwargs = self.compile_args(argv=argv, compile=compile, preprocessor=preprocessor, compiler_factory=compiler_factory)
+        return action(*args, **kwargs)
 
-            return args.action(*_args, **_kwargs)
 
 def factory(**kwargs):
     frame = sys._getframe()
